function [fX, dfX, loo] = kfdakernel(X, samples, labels);

% Gaussian kernel
% D+1 parameters
[n, D] = size(samples);         % number of examples and dimension of samples space
expX = exp(X);              % exponentiate the hyperparameters once and for all
gamma = 10;

% first, we write out the covariance matrix Q

Z = zeros(n,n);
for d = 1:D
  Z = Z + (repmat(samples(:,d),1,n)-repmat(samples(:,d)',n,1)).^2*expX(d);
end
Z = exp(-0.5*Z);
Z = [Z ones(length(labels),1)];
Q = Z'*Z + expX(D+1)*eye(n+1) + 1e-3*eye(n+1);                             % Gaussian term and noise


% then, we compute the smooth leave-one-out error ......
invQ = inv(Q);
invQZ = invQ*Z';
H = Z*invQZ;
delta = (labels - H*labels)./(1 - diag(H));
fX = mean((1-labels.*tanh(gamma*(labels - delta)))/2);
loo = mean((1-labels.*sign(labels - delta))/2);
term1 = gamma*labels.*(sech(gamma*(labels - delta))).^2/2/length(labels);

% fX = mean((tanh(gamma*(delta - 1)) + tanh(gamma*(-delta - 1)))/2 + 1);
% loo = mean((sign(delta - 1) + sign(-delta - 1))/2 + 1);
% term1 = gamma*(sech(gamma*(delta - 1)).^2 - sech(gamma*(-delta - 1)).^2)/length(labels)/2;
% ... and its partial derivatives

dfX = zeros(D+1,1);                     % set the size of the derivative vector

for d = 1:D
  V = -0.5*expX(d)*(repmat(samples(:,d),1,n)-repmat(samples(:,d)',n,1)).^2.*Z(:,1:n);
  V = [V zeros(n,1)];
  B = V*invQZ;
  HB = H*B;
  dfH = B + B' - HB - HB';
  dfX(d) = term1'*(-(dfH*labels)./(1 - diag(H)) + delta./(1 - diag(H)).*diag(dfH));
end 
% dfX(D+1) = term1'*(-invQ*(Z*invQt)./diaginvQ + delta./diaginvQ.*diag(invQ*Z*invQ));
dfH = -expX(D+1)*invQZ'*invQZ;
dfX(D+1) = term1'*((-dfH*labels)./(1 - diag(H)) + delta./(1 - diag(H)).*diag(dfH));