function [fX, dfX, loo] = mkfdakernel(X, samples, labels);
% Adaptive Scaling Gaussian kernel

[n, D] = size(samples);         % number of examples and dimension of samples space
expX = exp(X);              % exponentiate the hyperparameters once and for all
gamma = 10;
labelsnum = unique(labels);
c = length(labelsnum);
% first, we write out the covariance matrix Q

Z = zeros(n,n);
for d = 1:D
  Z = Z + (repmat(samples(:,d),1,n)-repmat(samples(:,d)',n,1)).^2*expX(d);
end
Z = exp(-0.5*Z);
Z = [Z ones(length(labels),1)];
Q = Z'*Z + expX(D+1)*eye(n+1) + 1e-4*eye(n+1);                             % Gaussian term and noise


% then, we compute the negative log likelihood ...
mlabels = zeros(n,c);
for i = 1:c
    tempindex1 = find(labels == labelsnum(i));
    tempindex2 = 1:n;
    tempindex2(tempindex1) = [];
    mlabels(tempindex1,i) = -1;
    mlabels(tempindex2,i) = 1;
end

invQ = inv(Q);
invQZ = invQ*Z';
H = Z*invQZ;

fX = 0;
loo = 0;
delta = zeros(n,c);
for i = 1:c
    delta(:,i) = (mlabels(:,i) - H*mlabels(:,i))./(1 - diag(H));
    fX = fX + mean((1 - mlabels(:,i).*tanh(gamma*(mlabels(:,i) - delta(:,i))))/2);
    loo = loo + mean((1 - mlabels(:,i).*sign(mlabels(:,i) - delta(:,i)))/2);
end
fX = fX/c;
loo = loo/c;

% ... and its partial derivatives
temp1 = zeros(n,c);
for i = 1:c
    term1(:,i) = gamma*mlabels(:,i).*(sech(gamma*(mlabels(:,i) - delta(:,i)))).^2/2/length(labels);
end

dfX = zeros(D+1,1);                     % set the size of the derivative vector

for d = 1:D
  V = -0.5*expX(d)*(repmat(samples(:,d),1,n)-repmat(samples(:,d)',n,1)).^2.*Z(:,1:n);
  V = [V zeros(n,1)];
  B = V*invQZ;
  HB = H*B;
  dfH = B + B' - HB - HB';
  for i = 1:c
      dfX(d) = dfX(d) + (term1(:,i))'*(-(dfH*mlabels(:,i))./(1 - diag(H)) + delta(:,i)./(1 - diag(H)).*diag(dfH));
  end
end 
% dfX(D+1) = term1'*(-invQ*(Z*invQt)./diaginvQ + delta./diaginvQ.*diag(invQ*Z*invQ));
dfH = -expX(D+1)*invQZ'*invQZ;
for i = 1:c
    dfX(D+1) = dfX(D+1) + (term1(:,i))'*((-dfH*mlabels(:,i))./(1 - diag(H)) + delta(:,i)./(1 - diag(H)).*diag(dfH));
end
dfX = dfX/c;